
import java.io.*;
import java.net.InetAddress;

public class ANEPacket {
    int version;
    int flags;
    int typeId;

    int hdrLen;
    int pktLen;

    TLVList TLVs;

    byte [] payload;

    static public int TAMANOIRtypeId = 20; // ???
    static public int PLANtypeId = 19;
    static public int ANTStypeId = 18;

    static public int curVersion = 1;
    static public int defaultFlags = 128;

    public ANEPacket (int version, int flags, int typeId,
		      int hdrLen, int pktLen,
		      TLVList TLVs,
		      byte [] payload) {
	this.version = version;
	this.flags = flags;
	this.typeId = typeId;
	this.hdrLen = hdrLen;
	this.pktLen = pktLen;
	this.TLVs = TLVs;
	this.payload = payload;
    }

    public ANEPacket (int typeId, TLVList TLVs, byte [] payload) {
	this.version = curVersion;
	this.flags = defaultFlags;
	this.typeId = typeId;
	this.TLVs = TLVs;
	this.payload = payload;

	this.hdrLen = TLVs.length () + 2;

	this.pktLen = hdrLen * 4 + payload.length;
    }

    public byte [] marshall () throws IOException {
	int i, curLen;
	TLVList temp = TLVs;
	ByteArrayOutputStream os = new ByteArrayOutputStream ();

	Marshalling.putByte (os, (byte) version);
	Marshalling.putByte (os, (byte) flags);

	Marshalling.putByte (os, (byte) (typeId / 256));
	Marshalling.putByte (os, (byte) (typeId % 256));

	Marshalling.putByte (os, (byte) (hdrLen / 256));
	Marshalling.putByte (os, (byte) (hdrLen % 256));

	Marshalling.putByte (os, (byte) (pktLen / 256));
	Marshalling.putByte (os, (byte) (pktLen % 256));



	curLen = 0;
	while (curLen < hdrLen-2) {
	    Marshalling.putTLV (os, temp.head ());
	    curLen += temp.head ().getLength ();
	    temp = temp.tail ();
	}

	for (i=0; i < pktLen - hdrLen*4; i++)
	    Marshalling.putByte (os, payload [i]);

	return os.toByteArray ();
    }

    public static ANEPacket unmarshall (byte [] data) 
	throws IOException, BadANEPversionException {

        int i,curLen;
        TLVList TLVs = TLVList.Nil;
        byte [] payload;

	InputStream is = new ByteArrayInputStream (data);

        int version = ((int) Marshalling.getByte (is) & 0xff);

	// verify that this is in sync with our expected version
	if (version != curVersion)
	  throw new BadANEPversionException(version,curVersion);

	// version is OK, we can continue with expected format
        int flags = ((int)Marshalling.getByte (is) & 0xff);
        int typeId = ((int)Marshalling.getByte (is) & 0xff);
        typeId = typeId * 256 + ((int)Marshalling.getByte (is) & 0xff);

        int hdrLen = ((int)Marshalling.getByte (is) & 0xff);
        hdrLen = hdrLen * 256 + ((int)Marshalling.getByte (is) & 0xff);

        int pktLen = ((int)Marshalling.getByte (is) & 0xff);
        pktLen = pktLen * 256 + ((int)Marshalling.getByte (is) & 0xff);

        curLen = 0;

        while (curLen < hdrLen-2) {         
            TLVs = TLVList.Cons (Marshalling.getTLV (is), TLVs);
            curLen += TLVs.head ().getLength ();
        }

        payload = new byte [pktLen - hdrLen*4];
        for (i=0; i < pktLen - hdrLen*4; i++)
            payload [i] = Marshalling.getByte (is);

        return new ANEPacket (version, flags, typeId, hdrLen, pktLen, TLVs, payload);
    }

    public static ANEPacket unmarshall (InputStream is) throws IOException {
        int i,curLen;
        TLVList TLVs = TLVList.Nil;
        byte [] payload;

        int version = ((int) Marshalling.getByte (is) & 0xff);
        int flags = ((int)Marshalling.getByte (is) & 0xff);
        int typeId = ((int)Marshalling.getByte (is) & 0xff);
        typeId = typeId * 256 + ((int)Marshalling.getByte (is) & 0xff);

        int hdrLen = ((int)Marshalling.getByte (is) & 0xff);
        hdrLen = hdrLen * 256 + ((int)Marshalling.getByte (is) & 0xff);

        int pktLen = ((int)Marshalling.getByte (is) & 0xff);
        pktLen = pktLen * 256 + ((int)Marshalling.getByte (is) & 0xff);

        curLen = 0;

        while (curLen < hdrLen-2) {         
            TLVs = TLVList.Cons (Marshalling.getTLV (is), TLVs);
            curLen += TLVs.head ().getLength ();
        }

        payload = new byte [pktLen - hdrLen*4];
        for (i=0; i < pktLen - hdrLen*4; i++)
            payload [i] = Marshalling.getByte (is);

        return new ANEPacket (version, flags, typeId, hdrLen, pktLen, TLVs, payload);
    }

    public int getVersion () { return version; }
    public int getFlags () { return flags; }
    public int getTypeId () { return typeId; }
    public int getHdrLen () { return hdrLen; }
    public int getPktLen () { return pktLen; }
    public TLVList getTLVs () { return TLVs; }
    public byte [] getPayload () { return payload; }

    private String byteToString (byte val) {
	String retval="";
	int i, b;

	b = (int) val & 0xff;
	for (i=0; i<7; i++) {
	    if ((b%2) == 1)
		retval = "1" + retval;
	    else retval = "0" + retval;
	    b = b >> 1;
	}

	return retval;
    }

    public String toString () {
	return
	    ("Version = " + version +
	     "\tFlags = " + byteToString ((byte)flags) +
	     "\tType ID = " + typeId);
    }
}
