/**
 * SendPackets.java : multithreaded stream server.
 *
 * @author Jean-Patrick Gelas
 */

import java.net.*;
import java.io.*;


//***************** envoi de paquets ********************

public class SendNaks extends Thread
{


//************* variables ********************

    static final String VERSION = "0.1";

    static final boolean DBG = true;
    
    static String serviceName = "FwdS"; // default service
    static int modulo = 100; // display info every 'modulo' sent packets
    static int delay  = 500; // default delay (ms) between each sent packet
    static String  proxyTAN = null;
    static int     proxyTANport = 5432; // default port
    static int     serverPort = 5432;   // default port
    static boolean nak = false;
    static int nak_size=100;
    // clients info
    String hostNameClient; // its name
    String proxyTANclient; // its proxyTAN
    int proxyTANportClient;// ...

    static int proto = TANet.TCP; // default protocol is TCP

    static String srcFileName = null ; // path for source file

    static int payloadSize = 4*1024; // default payload size
    static byte [] payload ;
    static final int SIZEOF_LONG = 8;

    // TCP variables -----------------------------
    static ServerSocket ss;
    Socket cc;            // current connection
    TANet net;


    // TCP constructor
    SendNak( Socket s )
    {
        cc = s;
    }
    
     
//************************ affiche usage *******************


    static void usage ()
    {
	System.out.println( "SendNaks - v " + VERSION );
	System.out.println( "------------------------");
	System.out.println( "Usage : SendNak [-h] [-size s] [-delay d] "+
			    "[-port p] [-sn service]");
	System.out.println( "        [-udp | -tcp] [-src f|-fix s] -tan "+
			    "proxyTAN");
	System.out.println( "");
	System.out.println( "");
	System.out.println( "  -h       : this help");
	System.out.println( "  -tan t   : this host must be connected to a proxyTAN");
	System.out.println( "  -size s  : payload size of ANEP packets");
	System.out.println( "  -delay d : delay (ms) between each sent packet");
	System.out.println( "  -port p  : send data on port p of proxyTAN");
	System.out.println( "  -src f   : data are read in file f.");
	System.out.println( "  -nak s   : data are always the same (use always the same ANEP packet).");
	System.out.println( "             If -src or -fix are omitted use stdin");
	System.out.println( "  -udp     : send data on a UDP channel");
	System.out.println( "  -tcp     : send data on a TCP channel (default)");
	System.out.println( "  -sn name : service name (default is FwdS)");
	System.out.println( "  -m i     : display info every i pkts received");
	System.out.println();
	System.out.println();
	System.out.println( "ex: ..." );
	System.out.println();
    } // usage()
  
  
  
  
  //************** gestion du passage des arguments ********************
  
  
  
    static void parseArgs ( String[]args )
    {
	if(args.length == 0) {
	    usage ();
	    System.exit (1);
	}
	int i;
	for (i = 0; i < args.length; i++)
	    {
		if (args[i].equals ("--help") || args[i].equals ("-h"))
		    {		
			usage ();
			System.exit (1);
		    }
		else if (args[i].equals ("-size"))
		    {		
			i++;
			payloadSize = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-delay"))
		    {		
			i++;
			delay = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-port"))
		    {		
			i++;
			proxyTANport = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-tan"))
		    {		
			i++;
			proxyTAN = args[i];
		    }
		else if (args[i].equals ("-udp"))
		    {		
			proto = TANet.UDP;
		    }
		else if (args[i].equals ("-tcp"))
		    {		
			proto = TANet.TCP;
		    }
		else if (args[i].equals ("-sn"))
		    {		
			i++;
			serviceName = args[i];
		    }
		else if (args[i].equals ("-src") || args[i].equals ("-f"))
		    {		
			i++;
			srcFileName = args[i];
		    }
		else if (args[i].equals ("-nak"))
		    {	
			i++;	
			nak = true;
			nak_size = Integer.parseInt(args[i]);
		    }
		else if (args[i].equals ("-m"))
                    {
                        i++;
                        modulo = Integer.parseInt(args[i]);
                    }
		else
		    {
			System.out.println ("Invalid argument : " + args[i]);
			usage ();
			System.exit (1);
		    }
	    }
    } // parseArgs()

    public static void displayOptions()
    {
	System.out.println ("Service Name : " + serviceName );
	System.out.println ("Delay        : " + delay );
	System.out.println ("proxy TAN    : " + proxyTAN );
	System.out.println ("port  TAN    : " + proxyTANport );
	System.out.print   ("source       : " );
	if(srcFileName == null) System.out.println("stdin");
	else System.out.println( srcFileName );
        System.out.print   ("Protocole    : " );
	switch(proto) {
	case TANet.UDP : System.out.println("UDP"); break;
	case TANet.TCP : System.out.println("TCP"); break;
	default : System.out.println( "unknown" );
	}
	System.out.println ("Payload size : " + payloadSize);
    }


//****************  envoi de paquet a partir d`un fichier *************
    public int sendFile() 
    {
	// cache some data
	byte [] srvNameinBytes    = serviceName.getBytes();
	byte [] destIdinBytes     = hostNameClient.getBytes(); 
	byte [] proxyTANIdinBytes = proxyTANclient.getBytes(); 

	byte [] srcIdinBytes      = net.srcId.getBytes();
        byte [] lastIdinBytes     = srcIdinBytes;

	// proxyTANportClient;
	
	
	
	
	

	try {
	    long t1,t2; // chrono variables
	    int cpt=0;
	    BufferedInputStream in = 
		new BufferedInputStream(new FileInputStream(srcFileName), 
					payloadSize);
	
	// si un fichier est specifie en entree par l`utilisateur, on le bufferise
	
	    int sz = 0, total_sz = 0;
	    t1 = System.currentTimeMillis();
            while((sz = in.read(payload)) > 0) {
		if(sz == payloadSize) {
		    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				    destIdinBytes, lastIdinBytes, 
				    proxyTANIdinBytes,
				    payload );
	 // creation d`un paquet et remplissage a partir du payload

		} else {
		    byte [] t = new byte[sz];
		    System.arraycopy(payload,0,t,0,sz);
		    if(DBG) System.out.println("SendPackets : Arghhh !!!");
		    net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				    destIdinBytes, lastIdinBytes, 
				    proxyTANIdinBytes, t );
		}
		// le paquet ne doit pas etre de la bonne taille, 
		//on passe un tableau de bits
		
		
	// ***********envoi du paquet anep ****************
		net.sendANEPkt();

		if( cpt++ % modulo == 0 )
		    System.out.println( "SendPackets : #" + cpt );
		total_sz += sz;
		try { Thread.sleep( delay ); } 
		catch (InterruptedException e) {
		}
	    } // while()
	    t2 = System.currentTimeMillis();
	    System.out.println( "SendPackets : " + (t2-t1) + "ms for " +
				total_sz + " Bytes." );

	} catch (FileNotFoundException e ) { 
	    System.out.println("SendPackets == " + e ); 
	} catch (EOFException e ) { 
	    System.err.println("SendPackets : EOF '"+ srcFileName +"'"); 
	} catch (IOException e ) { 
	    System.err.println("SendPackets := " + e );
	    System.exit(0);
	}
	return 0;
    } // sendFile()



//************* envoi de l`entree standard ********************

    // data source is stdin
    public int sendStdin()
    {
	// cache some data
	byte [] srvNameinBytes    = serviceName.getBytes();
	byte [] destIdinBytes     = hostNameClient.getBytes(); 
	byte [] proxyTANIdinBytes = proxyTANclient.getBytes(); 

	byte [] srcIdinBytes      = net.srcId.getBytes();
        byte [] lastIdinBytes     = srcIdinBytes;
	// proxyTANportClient;
	
	try {
	    int cpt=0;
	    BufferedInputStream in = new BufferedInputStream(System.in);
	    int sz = 0;
	    while((sz = in.read(payload)) > 0) {
		net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				destIdinBytes, lastIdinBytes, 
				proxyTANIdinBytes, 
				payload );
		net.sendANEPkt();
		
		if(DBG) if( cpt++ % modulo == 0 ) 
		    System.err.println( "SendPackets : #" + cpt );
		
		try { Thread.sleep( delay ); } 
		catch (InterruptedException e) {
		}
	    } // while()
	} catch (IOException e) {
	    System.err.println("SendPackets (sendStdin) : " + e);
	}
	return 0;
    } // sendStdin()



    //  ************************    sendNak ********************************
    
    
    public int sendNak()
    {
	// cache some data
	byte [] srvNameinBytes    = serviceName.getBytes();
	byte [] destIdinBytes     = hostNameClient.getBytes(); 
	byte [] proxyTANIdinBytes = proxyTANclient.getBytes(); 

	byte [] srcIdinBytes      = net.srcId.getBytes();
        byte [] lastIdinBytes     = srcIdinBytes;

	//ByteArrayOutputStream baos = new ByteArrayOutputStream();
	//DataOutputStream dos = new DataOutputStream( baos );
	
	

	try {
		System.out.println("proxyTANIdin="+proxyTANclient);
		System.out.println("srcIdinBytes="+srcIdinBytes);
		System.out.println("destIdinBytes ="+hostNameClient );	
		System.out.println("lastIdinBytes ="+lastIdinBytes );	
		System.out.println("serviceName ="+serviceName );
		System.out.println("srcIdinBytes ="+net.srcId );
					
	    	// remplissage du payload
	        
        	
        	for(int i=0; i<payloadSize;i++) payload[i] = 73;
		//payload=destIdinBytes;

		// creation d`un paquet
		net.makeANEPkt( srvNameinBytes, srcIdinBytes, 
				destIdinBytes, lastIdinBytes, 
				proxyTANIdinBytes,
				payload );
		//  envoi du paquet
		net.sendANEPkt();

		System.out.println( "Packet envoye " );
		
		if(delay != 0) try { 
		    Thread.sleep( delay ); } 
		catch (InterruptedException e) {
		}


	} catch (FileNotFoundException e ) { 
	    System.out.println("SendPackets == " + e ); 
	} catch (EOFException e ) { 
	    System.err.println("SendPackets : EOF : '"+ srcFileName +"'"); 
	} catch (IOException e ) { 
	    System.err.println("SendPackets = " + e );
	    System.exit(0);
	}
	return 0;
    } // sendNak()

    public void run()
    {
	try {
	    System.out.println("on est dans le run ");
	    // recover some informations from client (its name,
	    // proxyTANname where it is connected, and port)
	    
	    // recuperation du paquet et decoupage en donnee 
	    DataInputStream rd_cli = new DataInputStream(cc.getInputStream());

	    hostNameClient = rd_cli.readUTF();
	    proxyTANclient = rd_cli.readUTF();
	    proxyTANport   = rd_cli.readInt();
	    
	    if(DBG) System.out.println("SendPackets : (new cnx) client:"+
				       hostNameClient+" proxyTAN:"+
				       proxyTANclient+" ("+
				       proxyTANport+")"
				       );
	    rd_cli.close();
	   if (DBG) System.out.println("on ferme la connection");
	    cc.close(); // we don't use anymore this connection.
	} catch (IOException e) {
	    System.err.println( "SendPackets (tcp): " + e );
	}
	
	
	// Now we have to connect this thread to the closest proxyTAN
	// (given by arguments -tan hostname)
	if (DBG) System.out.println ("on se connect au proxyTAN le + proche");
	switch(proto) {
        case TANet.TCP:
	    net = new TANet( TANet.TCP, proxyTAN, proxyTANport );
	    break;
	
	case TANet.UDP:
	    net = new TANet( TANet.UDP, proxyTAN, proxyTANport );
	    break;
	}
	
	// and finally send data...
	if(srcFileName != null) {
	    System.out.println("send File... ");
	    //sendFix();
	    sendFile();
	    
	    
	} else {
	    if(nak) 
		sendNak();
	    else
		sendStdin();
	}		    
	net.close();
	if(net.ds != null) net.ds.close();
	if(DBG) System.out.println("SendPackets : run() thread done. /fin du run");
    } // run()
	

    public static void main( String [] args ) 
    {
	parseArgs( args );
	if(proxyTAN == null) {
	    usage (); System.exit (1);
	}

	payload = new byte[payloadSize];
	if(DBG) displayOptions();

	// wait for new clients
	try {
	    ss = new ServerSocket( serverPort );
	    System.out.println("SendPackets ( tcp ): waiting for connection"+
			       " on port " + serverPort);
	    while( true ) {
	    	System.out.println("on recupere le paquet de demande d emission");
		SendPackets sendp = new SendPackets( ss.accept() );
		System.out.println("on debute le thread");
		sendp.start(); // start thread
	    }
	} catch (IOException e) {
	    System.err.println( "-> SendPackets : " + e );
	}
    } // main()
}

