/*

  mitm.c

  Re-route packets with IP Router Alert option set to 'local' address.

  Display a message in /var/log/message when a TCP packet with IP
  Router Alert Option reach a node running this module.

  Change destination address (Man in the middle attack ?),
  and re-calculate the checksum.
  
  Auteur: Jean-Patrick Gelas
  Version: 0.01a
	code inspire de celui de RR (Rusty Russell) dans
	/usr/src/linux/net/ipv4/netfilter/ip_fw_compat.c
	et du noyau en general: ip.h, ip_options.c

	gcc -Wall -O2 -I/usr/src/linux/include -c module.c
	
  insmod module.o
  rmmod  module

  usage: insmod mitm.o local="10.0.0.1"

  'local' is the local IP address where the packets are now routed.

  This code is hooked on the NF_IP_PRE_ROUTING hook.

  compiled and runnable under 2.4.18 (w/ netfilter support)
*/

#define MODULE 
#define __KERNEL__

#include <linux/config.h>
#include <linux/types.h>
#include <linux/ip.h>
#include <linux/timer.h>
#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <net/protocol.h> 
#include <net/checksum.h> 

#include <linux/tcp.h>
#include <linux/udp.h>


#include "mitm.h"

#define DBG 


static int mytcpudp_csum(struct sk_buff *, unsigned int , unsigned int);
static unsigned int inet_addr(char *);
static void print_ip(unsigned int);



static int count_ipra = 0;
static unsigned int local_addr;
static char * local;

MODULE_DESCRIPTION("Mitm (prototype for Active Reliable Multicast)");
MODULE_PARM(local, "s");
MODULE_PARM_DESC(local, "Set local address"); /* objdump ??? */



static unsigned int ma_fonction(unsigned int hooknum,
				struct sk_buff **pskb,
				const struct net_device *in,
				const struct net_device *out,
				int (*okfn)(struct sk_buff *))
{
  unsigned int     prot ;
  int              optlen;
  unsigned char *  optptr;
 
  /* mitm variables */
  unsigned int osaddr, odaddr;
  unsigned int * dst_ip;
  
  /*****************************/
  /**code rajout par FBouhafs**/
  unsigned int * src_ip;
  local_addr = inet_addr(local);
 /**************************/  
  
  if (*pskb != NULL) {
    prot = SKB_PROTOCOL_IP (*pskb) ;  
    if(prot == TCP || prot == UDP) {
      optlen = SKB_LEN_IP(*pskb) - sizeof(struct iphdr);

      if(optlen > 0) { /* there is option(s) */
	optptr = (unsigned char *)(SKB_HEAD_IP  (*pskb)) + sizeof(struct iphdr);
	switch(*optptr) {
	case IPOPT_RA: /* Router Alert Option */
#ifdef DBG 
	  //printk("J'ai vu passer un gros minet (%d)\n", count_ipra);
	  count_ipra++;
#endif
	  osaddr = SKB_SADDR_IP(*pskb);
	  odaddr = SKB_DADDR_IP(*pskb);
	  
	  //local_addr = inet_addr("10.0.0.1");
	  

#ifdef DBG 
	  //printk("(faked)    odaddr = "); print_ip(local_addr);
	  dst_ip = &(SKB_DADDR_IP(*pskb));
	  src_ip = &(SKB_SADDR_IP(*pskb));
	  //printk("(avant)    dst_ip = "); print_ip(*dst_ip);
#endif
	  /********************************/
	  /**code rajout par FBouhafs*****/
	  /*il faut tester l@source du*****/
	  /*paquet pour ne pas intercepter*/
	  /****les paquets re-mis*********/ 
	  if(*src_ip!=local_addr)
	  *dst_ip = local_addr;
	  /*********************************/

#ifdef DBG 
	  //printk("(apres)    dst_ip = "); print_ip(*dst_ip);
	  //printk("(avant) csum = %d\n", SKB_CHECK_IP(*pskb));
#endif

	  ip_send_check((*pskb)->nh.iph);
	  mytcpudp_csum(*pskb, osaddr, odaddr);

#ifdef DBG 
	  //printk("(apres) csum = %d\n", SKB_CHECK_IP(*pskb));
#endif
	  break;
	}
      }
    }
  }
  return NF_ACCEPT;	
}



static struct nf_hook_ops preroute_ops = { 
	{ NULL, NULL }, 
	ma_fonction,		/* associate function */	
	PF_INET, 		/* IP protocol */
	NF_IP_PRE_ROUTING, 	/* hook */
	NF_IP_PRI_FILTER 	/* priority level */
};


static int __init init(void)
{
	int ret;
	
	ret = nf_register_hook(&preroute_ops);
	if (ret < 0) {
		printk("mitm: can't register pre-routing hook.\n");
		return ret;
	}

	printk("mitm: IP Router Alert detection + Mitm (local=%s)\n", local);


	return 0;
}

static void __exit fini(void)
{
 	printk("mitm: Module unloaded.\n");
	nf_unregister_hook(&preroute_ops);
}


/* My checksum function largely inpired by linux/net/ipv4/ip_nat_dumb.c */
static int mytcpudp_csum(struct sk_buff * skb, unsigned int osaddr, unsigned int odaddr )
{
  struct iphdr *iph = skb->nh.iph;
  u16     check;

  /* If it is the first fragment, rewrite protocol headers */
  if (!(iph->frag_off & htons(IP_OFFSET))) {
    u16     *cksum;

    switch(iph->protocol) {
    case IPPROTO_TCP:
      cksum  = (u16*)&((struct tcphdr*)(((char*)iph) + (iph->ihl<<2)))->check;
      if ((u8*)(cksum+1) > skb->tail)
	goto truncated;
      check = *cksum;
      if (skb->ip_summed != CHECKSUM_HW)
	check = ~check;
      check = csum_tcpudp_magic(iph->saddr, iph->daddr, 0, 0, check);
      check = csum_tcpudp_magic(~osaddr, ~odaddr, 0, 0, ~check);
      if (skb->ip_summed == CHECKSUM_HW)
	check = ~check;
      *cksum = check;
      break;
    case IPPROTO_UDP:
      cksum  = (u16*)&((struct udphdr*)(((char*)iph) + (iph->ihl<<2)))->check;
      if ((u8*)(cksum+1) > skb->tail)
	goto truncated;
      if ((check = *cksum) != 0) {
	check = csum_tcpudp_magic(iph->saddr, iph->daddr, 0, 0, ~check);
	check = csum_tcpudp_magic(~osaddr, ~odaddr, 0, 0, ~check);
	*cksum = check ? : 0xFFFF;
      }
      break;
    
    default:
      break;
    }
  }
  return NET_RX_SUCCESS;
  
 truncated:
  /* should be return NET_RX_BAD; */
  return -EINVAL;
}



/* change an IP char string in integer */
static unsigned int inet_addr(char *str)
{
  unsigned long l;
  unsigned int val;
  int i; l = 0;
  for (i =0; i < 4; i++)
    {
      l <<= 8;
      if(*str !='\0')
	{
	  val = 0;
	  while (*str != '\0' && *str != '.')
	    {
	      val *= 10;
	      val += *str - '0';
	      str++;
	    } l |= val;
	  if (*str != '\0')
	    str++;
	}
    } return(htonl(l));
}


/* display an integer IP (useful for debugging) */
static void print_ip(unsigned int addr) {
  printk("%d.%d.%d.%d\n", 
	 (addr)       & 0xff,
	 (addr >> 8)  & 0xff,
	 (addr >> 16) & 0xff,
	 (addr >> 24) & 0xff
	 );
}



module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");



/*
static void tcpudp_csum(struct sk_buff *x, unsigned int osaddr, unsigned int odaddr)
{
  struct iphdr *iph = x->nh.iph;
  unsigned short check =0x0000;
  unsigned short *cksum; 

  cksum = (unsigned short *)&((struct tcphdr *)(((char *)iph) +(iph->ihl<<2)))->check;
  check = csum_tcpudp_magic(iph->saddr, iph->daddr, 0, IPPROTO_TCP, ~(*cksum));
  *cksum = csum_tcpudp_magic(~osaddr, ~odaddr, 0, 0, ~check);
  *cksum += htons(IPPROTO_TCP);
}
*/
